function terminate=controlAlgorithm(t,Q)
% terminate=controlAlgorithm(t,Q)
%     Control Algorithm for MonkeyBot
%    in current setup, this is called every .001 seconds
%
%  ->Modifying global variable motorState allows control of motor torque
%  ->Modifiying magnetState allows changing of magnet configuration
%
%set terminate to 1 if integration should be halted (magnet transition, 
% motor change,  or if simulation is over)
%
%  This sample file does a simple open-loop controller.  It could be modified or
%  replaced 
%  
%  David Ufford
%  June 2009



%%%%MonkeyBot State Variables
global magnetState;%current state of magnet 
        %0 = magnets off, %1 = magnet 1 on.  %2= magnet 2 on %3=both
        %magnets on
global motorState;  %value of torque applied by motor

terminate=0;


%%%%Control Algoritm variables (these are specific to this control
%%%%algorimth)
%%% This open loop controller has only 1 cycle:
%  1.  Turns motor to max power for specified time
%  2.  Turns off motor, continues to swing
%  3.  Switches attached magnet at a second specified time
%  4.  Repeats cycle

global motor_t; %time motor is on each cycle
global swap_t; %time between magnet swaps (cycle time
global initial_t;  %time before first cycle starts
global maxMotorPower; %pull in max power of motor


%control tracking
global next_swap;  %time next swap will occur
if(~exist('next_swap') || size(next_swap,1)==0)
    if(initial_t>0)
        next_swap=initial_t;
        motorState=0;  %don't turn on motor till first cycle
    else
        next_swap=swap_t;
        motorState=maxMotorPower;
    end
    terminate=1;
end

if(t>next_swap-swap_t+motor_t && motorState~=0)
    %motor should be turned off
    motorState=0;
    terminate=1;
    %disp(['motor off at' num2str(t)]);
end



if(t>=next_swap)
    %reached time to swap attached magnets

    %swap magnet
    if(magnetState==1)
        magnetState=2;
    else
        magnetState=1;
    end
    %disp(['swapped at ' num2str(t)]);
    next_swap=next_swap+swap_t;  %set next cycle change
    motorState=maxMotorPower; %turn on motor for initial section

    terminate=1; %stop diffeq evaluation so that conditions can be reset
    %for the new constraints
end
