function terminate=controlAlgorithm_search(t,Q)
%terminate=controlAlgorithm_search(t,Q)
%
%  Modified version of control algorithm designed for gait searching.
%  1.  Impacts magnet when second link is at vertical
%  2.  Turns on motor until post-impact energy matches pre-impact energy
%  3.  Halts integration when link passes through vertical again
%
%  Also records timings of these events, stored in the global variables
%
% David Ufford
% June 2009


global magnetState;
global motorState;
terminate=0;


global motor_t; %time motor is turned off
global swap_t; %time motor is swapped
global final_t; %time integration is halted
global maxMotorPower;

global energy_goal; %energy we want to obtain after impact

global phase; %current gait stage we are ins

if(~exist('phase') || size(phase,1)==0 || phase<1)
    phase=1;%start at phase1
end

%Phase 1: -> Swing up

if(phase==1 && Q(7)<0) %%failed to reach vertical --- not a solution we are looking for
    terminate=1;
    swap_t=-1;
    motor_t=-1;
    phase=1;
    %disp(['failed vert at ' num2str(t)]);
    return;
end

%%If endpoint is at horizontal, swap magnets
%%i.e. t2=pi-2*t1;
if( phase==1 && (Q(4)- (pi-2*Q(3))) >=0 ) %use the zero crossing as the switch point
    swap_t=t;  %note the swap time
    motorState=maxMotorPower;%turn on motor
    energy_goal=calcEnergy2(Q);  %record pre-impact energy goal to reach
    phase=2;  %move to next phase
    terminate=1;
    % disp(['swp at ' num2str(t)]);
    return;
end

%%Phase 2: -> Motor Impulse

%%%If energy goal has been reached, turn off the motor
if(phase==2 && motorState~=0 && calcEnergy2(Q)>=energy_goal)
    motor_t=t; %record time of motor stop
    motorState=0;
    terminate=1;
    phase=3;
    %disp(['motor off at ' num2str(t)]);
    return;
end

%%Phase 3: -> Swing till back at vertical

%%Stop when t1=0
if(phase==3 && Q(3)>=0) % use zero crossing
    terminate=1;
    %disp(['stopped at ' num2str(t)]);
    final_t=t; %record time of cylce end

    %reset vars for next time around
    phase=1;
    energy_goal=0;
    motorState=0;

    return;
end

