%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Horizontal Gait Fitness Minization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Minimizes the values for dt1/dt2 to obtain the best horizontal gait 
%   fitness.  Runs the best dt1/dt2 through a basic stability check
%
%
%   David Ufford
%   June 2009

clear;
clear global;

%%
%%Initial Guess
%%
dt1_guess=.5;
dt2_guess=11.75;

%%%%%%%%%%%%%%%%
%%%%%%%System Parameters
global initial_t;
global motor_t;
global swap_t;
global maxMotorPower;
global doplot;
global final_t;


%start with set of parameters

maxMotorPower=2;

M(1)=3.083276839957533;
M(2)=3.083276839957533;
L(1)=0.610;
L(2)=0.610;
I(1)=5.386568898408416e-2;%M(1)*(L(1)+3*.01)^2/12;
I(2)=5.386568898408416e-2;%M(2)*(L(2)+3*.01)^2/12;
R(1)=L(1)-6.37687066165708e-2;%L(1)/2;
R(2)=6.37687066165708e-2;%L(2)/2;
g=9.81;

doplot=0;

%create function handle appropriate for the 'fminsearch' function
%this passes in all zeros for initial conditions except for the
%two parameters that should be provided in dt_i
min_fun=@(dt_i)runSim_fitness(...
    [0; 0; 0; 0; 0; 0; dt_i],20, M, I, L, R, g);

%set tolerance on initial conditions.
options=optimset('TolX',.0000001,'Display','iter');  

%%run matlabs's fminsearch to optimize
initial_conditions=fminsearch(min_fun, [dt1_guess; dt2_guess],options)

fitness=min_fun(initial_conditions) %run it again to get final fitness
initial_t=swap_t %and so we can grab the motor and magnet swap times
motor_t=motor_t-swap_t %then convert it into values used by the real control algorithm
swap_t=final_t

%%Stability
%run through several steps using the obtained initial conditions/timings
Q0=[zeros(6,1); initial_conditions];
stepLimit=17;
doplot=1; %watch it
[final_t, final_Q, step_y, step_fitness, step_energy]=runSim_stability(Q0,stepLimit, M, I, L, R, g);%call stability simulation


%%plot value results
figure(1);
plot(0:stepLimit,step_y(1:end-1));
title('Gait Deterioration');
xlabel('Iteration')
ylabel('Vertical Height at Impact');
figure(2)
plot(0:stepLimit,step_energy);
title('Gait Deterioration');
xlabel('Iteration')
ylabel('Kinetic Energy at Vertical');
figure(3)
plot(0:stepLimit,step_fitness);
title('Gait Deterioration');
xlabel('Iteration')
ylabel('Error Function at Vertical');
