function [fitness]=runSim_fitness(Q0,timeLimit, M, I, L, R, g)
% [fitness]=runSim_fitness(Q0,timeLimit, M, I, L, R, g)
%
%  Modified version of the primary simulation function, monkeySim.
% Runs the simulation through a single gait (from vertical, through impact,
% to vertical).  Calculates fitness/error and final point.
%
%  David Ufford
%  June 2009

global magnetState;%0 = magnets off, %1 = magnet 1 on.  %2= magnet 2 on
global motorState;%= value of torque to be applied
global terminated_t;
global terminated_Q;

DQcheck=@(t,Q,flag)DiffEqMonitor_search(t,Q,flag,M,I,L,R); %monitoring function
options = odeset('RelTol',1e-9,'AbsTol',1e-9,'OutputFcn',DQcheck);

t=0;  %current time
fixedEnd=1;  %currently Magnet 1 is on top
magnetState=1;  %and it is fixed to the place
motorState=0; %start with motor off
terminated_t=[];
terminated_Q=[];

%precalc some stuff

m2r2=M(2)*R(2)^2;
Mass_constant=[I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+m2r2,        m2r2+I(2);
    m2r2+I(2)       I(2)+m2r2];
Mass_var=[2*M(2)*R(2)*L(1), M(2)*R(2)*L(1);
    M(2)*R(2)*L(1) 0];
C_var=[-M(2)*R(2)*L(1)       -M(2)*R(2)*L(1);
    M(2)*R(2)*L(1)     0];
G_var=[g*(M(1)*R(1)+M(2)*L(1)) g*M(2)*R(2);
    0 g*M(2)*R(2)];

Q=Q0;

dt1_i=Q0(7); %intial values for comparison
dt2_i=Q0(8);

step=0;
%disp('Solving....');

while (t<timeLimit-.1)
    DiffEq=@(t,Q)DiffEqWrapper(t,Q,magnetState,Mass_constant,Mass_var,C_var,G_var);  %reduce number of parameters
    tspan=t:.001:timeLimit;
    [T, state]=ode45(DiffEq,tspan, Q,options); %--->GO!

    %%%%%%%%%OBTAIN CORRECT FINAL STATE
    if(terminated_t>0) %integration was halted due to controller change,
        t=terminated_t;
        Q=terminated_Q;   %get values at termination

        ii = find(T==terminated_t);
        T(ii:end)=[];  %remove extraneous values past the termination point
        state(ii:end,:)=[];

    else
        t=T(end); %obtain final values for next pass through ode45
        Q=state(end,:);
    end

    %%%%
    %%**********************
    %%%%Tracking as we progress through the gait
    %%************************


    %keep track of which "stage"/step we are in the gate.  The
    %controller_search function is setup to terminate at each step.
    step=step+1;


    if(step==1) %magnet impact
        %impact
        if (numel(Q)<4)
            keyboard %something bad happended
        end

        if(Q(7)<0) %link is going backward ... this is a bad solutoin
            fitness=25;
            %disp('didnt reach horiz')
            return;
        end

        %%%process impact
        %first, reposition variables so the other end is the base x/y
        [Q M I L R]=flipEnds(Q,M,I,L,R);

        dq=Q(5:8);
        P=buildP(Q,M,I,L,R);%.....projection matrix
        dq=P*dq;  %...calculate post-impact velocities
        Q=[Q(1:4); dq];
    elseif(step==2)
        %motor was turned off at this point
        %    ... just continue integration to the next step
    elseif(step==3)
        %Final point (end conditoin)
        fitness=(Q(7)-dt1_i)^2 + (Q(8)-dt2_i)^2 + Q(4)^2; %return fitness value
        return;
    end


end %loop until we reached the timeLimit for simulation
disp('Done!');


function P=buildP(Q,M,I,L,R)
%Builds the projection matrix used during impact

%parameters

t1=Q(3);
t2=Q(4);

%sins/cosines
C1=cos(t1);
C12=cos(t1+t2);
C2=cos(t2);
S1=sin(t1);
S12=sin(t1+t2);
S2=sin(t2);

mrml=M(1)*R(1)+M(2)*L(1);

Mass=[M(1)+M(2)   0   (mrml)*C1+M(2)*R(2)*C12  M(2)*R(2)*C12;
    0       M(1)+M(2)   (mrml)*S1+M(2)*R(2)*S12  M(2)*R(2)*S12;
    (mrml)*C1+M(2)*R(2)*C12          (mrml)*S1+M(2)*R(2)*S12 ...
    I(1)+I(2)+M(1)*R(1)^2+M(2)*L(1)^2+M(2)*R(2)^2+2*M(2)*R(2)*L(1)*C2         M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2);
    M(2)*R(2)*C12       M(2)*R(2)*S12       M(2)*R(2)^2+M(2)*R(2)*L(1)*C2+I(2)      I(2)+M(2)*R(2)^2];

invM=inv(Mass);

J=[1 0 0 0;
    0 1 0 0];

P=eye(4)-invM*(J.')*inv(J*invM*(J.'))*J;
