%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Horizontal Gait Fitness Search
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Iterates over given range of dt1/dt2, and calculates gait fitness at
%   each point.  Results saved to a .mat file and plotted in a 3D graph
%
%
%   David Ufford
%   June 2009


clear;
clear global;

%%
%%Search range
%%%%%%%%%%%%%
dt1_rng=0.05:.05:3;%range of values for dt1 to check
dt2_rng=8:.05:20;%range of values for dt2 to check
%%%%
%%
%%

%%%%%%%%%%%%%%%%
%%%%%%%System Parameters
global initial_t;
global motor_t;
global swap_t;
global maxMotorPower;
global doplot;
global final_t;


maxMotorPower=2;

M(1)=3.083276839957533;
M(2)=3.083276839957533;
L(1)=0.610;
L(2)=0.610;
I(1)=5.386568898408416e-2;
I(2)=5.386568898408416e-2;
R(1)=L(1)-6.37687066165708e-2;
R(2)=6.37687066165708e-2;
g=9.81;

disp(['dt1 range: ' num2str(dt1_rng(1)) ' - ' num2str(dt1_rng(end))]);
disp(['dt2 range: ' num2str(dt2_rng(1)) ' - ' num2str(dt2_rng(end))]);


%create shorter handle to function that generates the error value
%this passes in all zeros for initial conditions except for the
%two parameters that should be provided in dt_i
fitnessFunction=@(dt_i)runSim_fitness(...
    [0; 0; 0; 0; 0; 0; dt_i],20, M, I, L, R, g);


%%initialize storage arrays
dt1_size=length(dt1_rng);
dt2_size=length(dt2_rng);
swapt=zeros(dt1_size,dt2_size);
motort=zeros(dt1_size,dt2_size);
dt1val=zeros(dt1_size,dt2_size);
dt2val=zeros(dt1_size,dt2_size);
result=ones(dt1_size, dt2_size)*25;

doplot=0;%don't display any graphs

totalcount=dt1_size*dt2_size;
i=0; i1=0; i2=0;%counting indeces for progress

avgtime=0;

starttime=cputime;
tmleft=0;

for dt1=dt1_rng;%iterate through all dt1 values
    i1=i1+1;
    i2=0;
    for dt2=dt2_rng;%iterate through all dt2 values
        i2=i2+1;
        dt1val(i1,i2)=dt1;
        dt2val(i1,i2)=dt2;

        clear global magnetState; %clear the states between each run
        clear global motorState;
        swap_t=0;

        %run the simulation
        [err_val]=fitnessFunction([dt1; dt2]);%call fitness function with initial conditions

        if(swap_t>0) %check to make sure we had enough energy to reach the y=0 point
            %at this final point (after swap, at t1=0)

            %store results
            motort(i1,i2)=motor_t;
            swapt(i1,i2)=swap_t;
            fint(i1,i2)=final_t(end);

            result(i1,i2)=err_val; %record fitness in array
        end

        if(mod(i2,100)==1)
            %(broken) time tracking
            sofar=(i1-1)*dt2_size+i2;
            tmleft=(cputime-starttime)/sofar * (totalcount-sofar)/60;
            fprintf('current dt1/dt2: %0.3f/%0.3f, guess at timeleft= %f mins (%i/%i)\n', dt1,dt2,tmleft,sofar,totalcount);
        end
    end
end

save(['run_' datestr(now,'HHMMSS')]);%save everything to a mat file


result=sqrt(result);
result(result>5)=5;%modify the results for better visibility

%-->PLOT
surf(dt2_rng, dt1_rng, result)
shading interp
colormap(hot)
xlabel('dt2');
ylabel('dt1');
zlabel('Sqrt(Error)');
title('Horizontal Gait Fitness');
